package com.heng.wechat;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.text.TextUtils;
import java.util.ArrayList;
import android.content.Intent;

import com.tencent.mm.sdk.modelbase.BaseReq;
import com.tencent.mm.sdk.modelbase.BaseResp;
import com.tencent.mm.sdk.constants.ConstantsAPI;
import com.facebook.react.bridge.Callback;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableMap;
import com.tencent.mm.sdk.modelmsg.SendAuth;
import com.tencent.mm.sdk.modelmsg.SendMessageToWX;
import com.tencent.mm.sdk.modelmsg.WXImageObject;
import com.tencent.mm.sdk.modelmsg.WXMediaMessage;
import com.tencent.mm.sdk.modelmsg.WXMusicObject;
import com.tencent.mm.sdk.modelmsg.WXTextObject;
import com.tencent.mm.sdk.modelmsg.WXVideoObject;
import com.tencent.mm.sdk.modelmsg.WXWebpageObject;
import com.tencent.mm.sdk.modelpay.PayReq;
import com.tencent.mm.sdk.openapi.IWXAPI;
import com.tencent.mm.sdk.openapi.WXAPIFactory;
import com.tencent.mm.sdk.openapi.IWXAPIEventHandler;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import com.facebook.react.modules.core.DeviceEventManagerModule;

import com.switfpass.pay.MainApplication;
import com.switfpass.pay.activity.PayPlugin;
import com.switfpass.pay.bean.RequestMsg;
import com.switfpass.pay.service.GetPrepayIdResult;
import com.switfpass.pay.utils.MD5;
import com.switfpass.pay.utils.SignUtils;
import com.switfpass.pay.utils.Util;
import com.switfpass.pay.utils.XmlUtils;

import android.app.Activity;  
import android.util.Log;
/**
 * Created by heng on 2015/12/10.
 * <p/>
 * Edited by heng on 15/12/18
 * Added share webPage and weChat Pay
 * <p/>
 * Edited by heng on 2015/12/22
 * Add remote image async download
 * <p/>
 * Edited by heng on 2015/12/29
 * 1.Removed Handler and Thread
 * 2.Modify options param
 * 3.Added share local
 * 4.Added and remote image(分享远程图片到朋友圈和收藏都会失败,具体原因待查,建议把远程图片下载到本地来分享)
 * <p/>
 * Edited by heng on 2016/02/02
 * 1.Added method openWXApp
 * 2.Edited callback(err,res)
 * 3.Reconstruction code
 */
public class WeChatModule extends ReactContextBaseJavaModule implements IWXAPIEventHandler{

    public static final String REACT_MODULE_NAME = "WeChatAndroid";

    private IWXAPI wxApi = null;
    private String appId;
    private ReactApplicationContext reactApplicationContext;

    /*============ WeChat share options key ==============*/
    public static final String OPTIONS_TITLE = "title";
    public static final String OPTIONS_DESC = "description";
    public static final String OPTIONS_TAG_NAME = "tagName";
    public static final String OPTIONS_THUMB_SIZE = "thumbSize";
    public static final String OPTIONS_TRANSACTION = "transaction";
    public static final String OPTIONS_SCENE = "shareWay";
    public static final String OPTIONS_TYPE = "cmd";

    public static final String OPTIONS_TEXT = "text";

    public static final String OPTIONS_IMAGE_URL = "imageUrl";
    public static final String OPTIONS_IMAGE_PATH = "imagePath";

    public static final String OPTIONS_THUMB_IMAGE = "thumbImage";

    public static final String OPTIONS_WEBPAGE_URL = "webpageUrl";

    public static final String OPTIONS_MUSIC_URL = "musicUrl";
    public static final String OPTIONS_MUSIC_LOW_BAND_URL = "musicLowBandUrl";

    public static final String OPTIONS_VIDEO_URL = "videoUrl";
    public static final String OPTIONS_VIDEO_LOW_BAND_URL = "videoLowBandUrl";
    /*============ WeChat share options key ==============*/
    public static final String OPTIONS_APP_ID = "appId";
    public static final String OPTIONS_NONCE_STR = "nonceStr";
    public static final String OPTIONS_PACKAGE_VALUE = "packageValue";
    public static final String OPTIONS_PARTNER_ID = "partnerId";
    public static final String OPTIONS_PREPAY_ID = "prepayId";
    public static final String OPTIONS_TIME_STAMP = "timeStamp";
    public static final String OPTIONS_SIGN = "sign";
    public static final String OPTIONS_TOKEN_ID = "tokenId";
    /*============ WeChat pay options key ==============*/
    public static final String TYPE_TEXT = "TextShare";          //文字
    public static final String TYPE_IMAGE = "ImageShare";         //图片
    public static final String TYPE_WEB_PAGE = "WebShare";      //网页
    public static final String TYPE_MUSIC = "MusicShare";         //音乐
    public static final String TYPE_VIDEO = "VideoShare";         //视频

    String tagName = null;
    String title = null;
    String desc = null;
    String transaction = null;
    Bitmap bitmap = null;           //分享的缩略图
    int thumbSize = 150;            //分享的缩略图大小
    int scene;                      //分享的方式(0:聊天界面，1:朋友圈，2:收藏)

    private Activity mCurrentActivity;
    public WeChatModule(ReactApplicationContext reactContext,Activity activity) {
        super(reactContext);
        reactApplicationContext = reactContext;
        mCurrentActivity=activity;
    }

    @Override
    public String getName() {
        return REACT_MODULE_NAME;
    }

    private static ArrayList<WeChatModule> modules = new ArrayList<>();

    @Override
    public void initialize() {
        super.initialize();
        modules.add(this);
    }

    @Override
    public void onCatalystInstanceDestroy() {
        super.onCatalystInstanceDestroy();
        if (wxApi != null) {
            wxApi = null;
        }
        modules.remove(this);
    }

    public static void handleIntent(Intent intent) {
        for (WeChatModule mod : modules) {
            mod.wxApi.handleIntent(intent, mod);
        }
    }

    private void sendEvent(String eventName,String returnMsg){
        this.getReactApplicationContext()
            .getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
            .emit(eventName, returnMsg);
    }

    /**
     * 注册AppID到微信（使用微信SDK必须先调用此方法）
     */
    @ReactMethod
    public void registerApp(String appId, Callback callback) {
        if (TextUtils.isEmpty(appId)) {
            if (callback != null) {
                callback.invoke("appId must be not null !");
            }
        }
        else if(wxApi == null){
            this.appId = appId;
            wxApi = WXAPIFactory.createWXAPI(getReactApplicationContext(), appId, true);
            boolean registered = wxApi.registerApp(appId);
            if (callback != null) {
                callback.invoke(null, registered);
            }
        }
    }

    private void commonCallback(Callback callback, boolean res) {
        if (callback != null) {
            if (wxApi == null) {
                callback.invoke("please registerApp before this !");
                return;
            }
            callback.invoke(null, res);
        }
    }

    /**
     * 打开微信客户端
     */
    @ReactMethod
    public void openWXApp(Callback callback) {
        commonCallback(callback, wxApi.openWXApp());
    }

    /**
     * 判断是否安装微信
     */
    @ReactMethod
    public void isWXAppInstalled(Callback callback) {
        commonCallback(callback, wxApi.isWXAppInstalled());
    }

    /**
     * 判断安装的版本是否为微信支持的API
     */
    @ReactMethod
    public void isWXAppSupportAPI(Callback callback) {
        commonCallback(callback, wxApi.isWXAppSupportAPI());
    }

    /**
     * 获取微信支持的API版本
     */
    @ReactMethod
    public void getWXAppSupportAPI(Callback callback) {
        if (callback != null) {
            if (wxApi == null) {
                callback.invoke("please registerApp before this !");
                return;
            }
            int supportAPI =wxApi.getWXAppSupportAPI();
            callback.invoke(null, supportAPI);
        }
    }

    /**
     * 微信授权登录
     */
    @ReactMethod
    public void sendAuthReq(String scope, String state, Callback callback) {
        if (wxApi == null) {
            if (callback != null) {
                callback.invoke("please registerApp before this !");
            }
        } else {
            if (TextUtils.isEmpty(scope)) {
                scope = "snsapi_userinfo";
            }
            if (TextUtils.isEmpty(state)) {
                state = "SECRET";
            }
            SendAuth.Req req = new SendAuth.Req();
            req.scope = scope;
            req.state = state;
            boolean sendReqOK =wxApi.sendReq(req);
            if (callback != null) {
                callback.invoke(null, sendReqOK);
            }
        }
    }

    /**
     * 分享到微信
     */
    @ReactMethod
    public void sendReq(ReadableMap options, Callback callback) {
        if (wxApi == null) {
            if (callback != null) {
                callback.invoke("please registerApp before this !");
            }
        } else {
            if (options == null) {
                if (callback != null) {
                    callback.invoke("please setting options !");
                }
            } else {
                if (options.hasKey(OPTIONS_TYPE)) {
                    String type = options.getString("cmd");
                    if(type.equals("WebShare")){
                        WXWebpageObject webpage = new WXWebpageObject();
                        webpage.webpageUrl = options.getString("webpageUrl");
                        WXMediaMessage msg = new WXMediaMessage(webpage);
                        msg.title = options.getString("title");
                        msg.description = options.getString("description");

                        SendMessageToWX.Req req = new SendMessageToWX.Req();
                        req.transaction = String.valueOf(System.currentTimeMillis());
                        req.message = msg;
                        req.scene = SendMessageToWX.Req.WXSceneSession;
                        boolean sendReqOK = wxApi.sendReq(req);
                        if (callback != null) {
                            callback.invoke(null, sendReqOK);
                        }
                    }

                    // }else if(type.equals(TYPE_IMAGE)){
                    //     msg.mediaObject = getImageObj(options);
                    // }else if(type.equals(TYPE_WEB_PAGE)){

                    //     msg.mediaObject = getWebpageObj(options);
                    // }else if(type.equals(TYPE_MUSIC)){
                    //     msg.mediaObject = getMusicObj(options);
                    // }else if(type.equals(TYPE_VIDEO)){
                    //     msg.mediaObject = getVideoObj(options);
                    // }else{
                    //     if (callback != null) {
                    //             callback.invoke("please check correct media type !");
                    //         }
                    // }
                    // if (options.hasKey(OPTIONS_TITLE)) {
                    //     title = options.getString(OPTIONS_TITLE);
                    // }
                    // if (options.hasKey(OPTIONS_DESC)) {
                    //     desc = options.getString(OPTIONS_DESC);
                    // }
                    // if (options.hasKey(OPTIONS_TAG_NAME)) {
                    //     tagName = options.getString(OPTIONS_TAG_NAME);
                    // }
                    // if (options.hasKey(OPTIONS_THUMB_SIZE)) {
                    //     thumbSize = options.getInt(OPTIONS_THUMB_SIZE);
                    // }
                    // if (options.hasKey(OPTIONS_TRANSACTION)) {
                    //     transaction = options.getString(OPTIONS_TRANSACTION);
                    // }
                    // if (options.hasKey(OPTIONS_SCENE)) {
                    //     scene = options.getInt(OPTIONS_SCENE);
                    // }

                    // if (!TextUtils.isEmpty(title)) {
                    //     msg.title = title;
                    // }
                    // if (!TextUtils.isEmpty(desc)) {
                    //     msg.description = desc;
                    // }
                    // if (!TextUtils.isEmpty(tagName)) {
                    //     msg.mediaTagName = tagName;
                    // }
                    // if (bitmap != null) {
                    //     Bitmap thumbBmp = Bitmap.createScaledBitmap(bitmap, thumbSize, thumbSize, true);
                    //     bitmap.recycle();
                    //     msg.thumbData = bmpToByteArray(thumbBmp, true);
                    // }

                    // SendMessageToWX.Req req = new SendMessageToWX.Req();
                    // req.message = msg;
                    // if (!TextUtils.isEmpty(transaction)) {
                    //     req.transaction = transaction;
                    // } else {
                    //     req.transaction = String.valueOf(System.currentTimeMillis());
                    // }
                    // if (scene == 0 || scene == 1 || scene == 2) {
                    //     req.scene = scene;
                    // } else {
                    //     req.scene = 0;
                    // }
                    // boolean sendReqOK = wxApi.sendReq(req);
                    // if (callback != null) {
                    //     callback.invoke(null, sendReqOK);
                    // }
                } else {
                    if (callback != null) {
                        callback.invoke("please setting share type !");
                    }
                }
            }
        }
    }


    /**
     * 微信支付
     */
    @ReactMethod
    public void weChatPay(ReadableMap options,Callback callback) {
        // if (wxApi == null) {
        //     if (callback != null) {
        //         callback.invoke("please registerApp before this !");
        //     }
        //     return;
        // }
        String appId ="";
        String tokenId="";
        //final Activity currentActivity = getCurrentActivity();
        // String nonceStr = "";
        // String packageValue = "";
        // String partnerId = "";
        // String prepayId = "";
        // String timeStamp = "";
        // String sign = "";
        // if (options != null) {
        //     if (options.hasKey(OPTIONS_APP_ID)) {
        //         appId = options.getString(OPTIONS_APP_ID);
        //     }
        //     if (options.hasKey(OPTIONS_NONCE_STR)) {
        //         nonceStr = options.getString(OPTIONS_NONCE_STR);
        //     }
        //     if (options.hasKey(OPTIONS_PACKAGE_VALUE)) {
        //         packageValue = options.getString(OPTIONS_PACKAGE_VALUE);
        //     }
        //     if (options.hasKey(OPTIONS_PARTNER_ID)) {
        //         partnerId = options.getString(OPTIONS_PARTNER_ID);
        //     }
        //     if (options.hasKey(OPTIONS_PREPAY_ID)) {
        //         prepayId = options.getString(OPTIONS_PREPAY_ID);
        //     }
        //     if (options.hasKey(OPTIONS_TIME_STAMP)) {
        //         timeStamp = options.getString(OPTIONS_TIME_STAMP);
        //     }
        //     if (options.hasKey(OPTIONS_SIGN)) {
        //         sign = options.getString(OPTIONS_SIGN);
        //     }
        // }

        // PayReq request = new PayReq();
        // request.appId = appId;
        // request.nonceStr = nonceStr;
        // request.packageValue = packageValue;
        // request.partnerId = partnerId;
        // request.prepayId = prepayId;
        // request.timeStamp = timeStamp;
        // request.sign = sign;
        // boolean sendReqOK = wxApi.sendReq(request);
        // if (callback != null) {
        //     callback.invoke(null, sendReqOK);
        // }9+
        if (options != null) {
            if (options.hasKey(OPTIONS_APP_ID)) {
                appId = options.getString(OPTIONS_APP_ID);
            }
            if (options.hasKey(OPTIONS_TOKEN_ID)) {
                tokenId = options.getString(OPTIONS_TOKEN_ID);
            }
        }
        RequestMsg msg = new RequestMsg();
        msg.setTokenId(tokenId);
        msg.setTradeType(MainApplication.WX_APP_TYPE);
        msg.setAppId(appId);
        PayPlugin.unifiedAppPay(mCurrentActivity, msg);
    }


    /**
     * Added by heng on 2015/12/29
     * <p/>
     * Bitmap to byte array
     */
    private byte[] bmpToByteArray(Bitmap bmp, boolean needRecycle) {
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        bmp.compress(Bitmap.CompressFormat.PNG, 100, output);
        if (needRecycle) {
            bmp.recycle();
        }
        byte[] result = output.toByteArray();
        try {
            output.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 获取文本对象
     * */
    private WXTextObject getTextObj(ReadableMap options) {
        WXTextObject textObject = new WXTextObject();
        if (options.hasKey(OPTIONS_TEXT)) {
            textObject.text = options.getString(OPTIONS_TEXT);
        }
        return textObject;
    }

    /**
     * 获取图片对象
     * */
    private WXImageObject getImageObj(ReadableMap options) {
        WXImageObject imageObject = new WXImageObject();
        if (options.hasKey(OPTIONS_IMAGE_URL)) {
            String remoteUrl = options.getString(OPTIONS_IMAGE_URL);
            imageObject.imageUrl = remoteUrl;
            try {
                bitmap = BitmapFactory.decodeStream(new URL(remoteUrl).openStream());
            } catch (IOException e) {
                bitmap = null;
                e.printStackTrace();
            }
        }
        if (options.hasKey(OPTIONS_IMAGE_PATH)) {
            String localPath = options.getString(OPTIONS_IMAGE_PATH);
            File file = new File(localPath);
            if (file.exists()) {
                imageObject.setImagePath(localPath);
                bitmap = BitmapFactory.decodeFile(localPath);
            } else {
                bitmap = null;
            }
        }
        return imageObject;
    }

    /**
     * 获取网页对象
     * */
    private WXWebpageObject getWebpageObj(ReadableMap options) {
        WXWebpageObject webpageObject = new WXWebpageObject();
        if (options.hasKey(OPTIONS_WEBPAGE_URL)) {
            webpageObject.webpageUrl = options.getString(OPTIONS_WEBPAGE_URL);
        }
        if(options.hasKey(OPTIONS_THUMB_IMAGE)){
            String thumbImage = options.getString(OPTIONS_THUMB_IMAGE);
            try {
                bitmap = BitmapFactory.decodeStream(new URL(thumbImage).openStream());
            } catch (IOException e) {
                bitmap = null;
                e.printStackTrace();
            }
        }
        return webpageObject;
    }

    /**
     * 获取音乐对象
     * */
    private WXMusicObject getMusicObj(ReadableMap options) {
        WXMusicObject musicObject = new WXMusicObject();
        if (options.hasKey(OPTIONS_MUSIC_URL)) {
            musicObject.musicUrl = options.getString(OPTIONS_MUSIC_URL);
        }
        if (options.hasKey(OPTIONS_MUSIC_LOW_BAND_URL)) {
            musicObject.musicLowBandUrl = options.getString(OPTIONS_MUSIC_LOW_BAND_URL);
        }

        if(options.hasKey(OPTIONS_THUMB_IMAGE)){
            String thumbImage = options.getString(OPTIONS_THUMB_IMAGE);
            try {
                bitmap = BitmapFactory.decodeStream(new URL(thumbImage).openStream());
            } catch (IOException e) {
                bitmap = null;
                e.printStackTrace();
            }
        }
        return musicObject;
    }

    /**
     * 获取视频对象
     * */
    private WXVideoObject getVideoObj(ReadableMap options){
        WXVideoObject videoObject = new WXVideoObject();
        if(options.hasKey(OPTIONS_VIDEO_URL)){
            videoObject.videoUrl = options.getString(OPTIONS_VIDEO_URL);
        }
        if(options.hasKey(OPTIONS_VIDEO_LOW_BAND_URL)){
            videoObject.videoLowBandUrl = options.getString(OPTIONS_VIDEO_LOW_BAND_URL);
        }
        if(options.hasKey(OPTIONS_THUMB_IMAGE)){
            String thumbImage = options.getString(OPTIONS_THUMB_IMAGE);
            try {
                bitmap = BitmapFactory.decodeStream(new URL(thumbImage).openStream());
            } catch (IOException e) {
                bitmap = null;
                e.printStackTrace();
            }
        }
        return videoObject;
    }

    @Override
    public void onReq(BaseReq baseReq) {

    }

    @Override
    public void onResp(BaseResp baseResp) {
        Log.v("WXEntryActivity",baseResp.errCode+"");
        String eventName=null;
        String message=null;
        if (baseResp instanceof SendAuth.Resp) {
            SendAuth.Resp resp = (SendAuth.Resp) (baseResp);
            if (resp.errCode == BaseResp.ErrCode.ERR_OK) {
                // map.putString("type", "SendAuth.Resp");
                // map.putString("code", resp.code);
                // map.putString("state", resp.state);
                // map.putString("url", resp.url);
                // map.putString("lang", resp.lang);
                // map.putString("country", resp.country);
                eventName="finishedAuth";
                message=resp.code;
            }
        } else if (baseResp instanceof SendMessageToWX.Resp) {
            SendMessageToWX.Resp resp = (SendMessageToWX.Resp) (baseResp);
            //map.putString("type", "SendMessageToWX.Resp");
            eventName="finishedShare";
            message="success";
        }else if(baseResp.getType()==ConstantsAPI.COMMAND_PAY_BY_WX){
            if (baseResp.errCode == 0){
              eventName="finishedPay";
              message="success";
            }
        }
        sendEvent(eventName,message);
    }
}
